from decimal import Decimal
import unittest

from django_easyfilters.ranges import auto_ranges


class TestRanges(unittest.TestCase):

    def test_auto_ranges_simple(self):
        """
        Test that auto_ranges produces 'nice' looking automatic ranges.
        """
        # An easy case - max_items is just what we want
        ranges1 = auto_ranges(Decimal('15.0'), Decimal('20.0'), 5)
        self.assertEqual(ranges1,
                         [(Decimal('15.0'), Decimal('16.0')),
                          (Decimal('16.0'), Decimal('17.0')),
                          (Decimal('17.0'), Decimal('18.0')),
                          (Decimal('18.0'), Decimal('19.0')),
                          (Decimal('19.0'), Decimal('20.0'))])

    def test_auto_ranges_flexible_max_items(self):
        # max_items is a bit bigger than what we want,
        # but we should be flexible if there is an easy target.
        ranges1 = auto_ranges(Decimal('15.0'), Decimal('20.0'), 6)
        self.assertEqual(ranges1,
                         [(Decimal('15.0'), Decimal('16.0')),
                          (Decimal('16.0'), Decimal('17.0')),
                          (Decimal('17.0'), Decimal('18.0')),
                          (Decimal('18.0'), Decimal('19.0')),
                          (Decimal('19.0'), Decimal('20.0'))])

    def test_auto_ranges_round_limits(self):
        # start and end limits should be rounded to something nice

        # Check with 5-10, 50-100, 15-20, 150-200

        ranges1 = auto_ranges(Decimal('15.1'), Decimal('19.9'), 5)
        self.assertEqual(ranges1,
                         [(Decimal('15.0'), Decimal('16.0')),
                          (Decimal('16.0'), Decimal('17.0')),
                          (Decimal('17.0'), Decimal('18.0')),
                          (Decimal('18.0'), Decimal('19.0')),
                          (Decimal('19.0'), Decimal('20.0'))])

        ranges2 = auto_ranges(Decimal('151'), Decimal('199'), 5)
        self.assertEqual(ranges2,
                         [(Decimal('150'), Decimal('160')),
                          (Decimal('160'), Decimal('170')),
                          (Decimal('170'), Decimal('180')),
                          (Decimal('180'), Decimal('190')),
                          (Decimal('190'), Decimal('200'))])

        ranges3 = auto_ranges(Decimal('5.1'), Decimal('9.9'), 5)
        self.assertEqual(ranges3,
                         [(Decimal('5.0'), Decimal('6.0')),
                          (Decimal('6.0'), Decimal('7.0')),
                          (Decimal('7.0'), Decimal('8.0')),
                          (Decimal('8.0'), Decimal('9.0')),
                          (Decimal('9.0'), Decimal('10.0'))])

        ranges4 = auto_ranges(Decimal('51'), Decimal('99'), 5)
        self.assertEqual(ranges4,
                         [(Decimal('50'), Decimal('60')),
                          (Decimal('60'), Decimal('70')),
                          (Decimal('70'), Decimal('80')),
                          (Decimal('80'), Decimal('90')),
                          (Decimal('90'), Decimal('100'))])

        ranges5 = auto_ranges(Decimal('3'), Decimal('6'), 5)
        self.assertEqual(ranges5,
                         [(Decimal('3'), Decimal('4')),
                          (Decimal('4'), Decimal('5')),
                          (Decimal('5'), Decimal('6'))])

    def test_auto_ranges_type(self):
        """
        auto_ranges should return the same type of thing it is passed
        """
        r = auto_ranges(1, 10, 10)
        self.assertEqual(type(r[0][0]), int)

        r2 = auto_ranges(Decimal('1'), Decimal('10'), 10)
        self.assertEqual(type(r2[0][0]), Decimal)



"""
Utilities to produce ranges of values for filters
"""

from decimal import Decimal, DecimalTuple, ROUND_HALF_EVEN, ROUND_DOWN, ROUND_UP
import math


def round_dec(d):
    return d._rescale(0, ROUND_HALF_EVEN)

def round_dec_down(d):
    return d._rescale(0, ROUND_DOWN)

def round_dec_up(d):
    return d._rescale(0, ROUND_UP)


def auto_ranges(lower, upper, max_items):
    if lower == upper:
        return [(lower, upper)]

    assert lower < upper

    input_type = type(lower)

    # Convert to decimals.
    lower_d = Decimal(lower)
    upper_d = Decimal(upper)

    step = (upper_d - lower_d)/max_items

    # For human presentable numbers, 'step' will preferable be something like 2,
    # 5, 10, or 0.1, 0.2, 0.5. etc.
    st = step.as_tuple()

    # It's not very helpful having exponent > 0, because these get displayed as
    # 1E+1 etc. But we also don't want things like 10.00000. So we make sure
    # we don't increase the exponent of step above zero

    exponent_offset = len(st.digits)
    if exponent_offset + st.exponent > 0:
        exponent_offset = -st.exponent
    zeros = [0] * (len(st.digits) - 1 - exponent_offset)

    candidate_steps = [Decimal(DecimalTuple(sign=st.sign,
                                            digits=[d] + zeros,
                                            exponent=st.exponent + exponent_offset))
                       for d in (1, 2, 5)]
    # Go one order bigger as well:
    candidate_steps.append(Decimal(DecimalTuple(sign=st.sign,
                                                digits=[1, 0] + zeros,
                                                exponent=st.exponent + exponent_offset)))

    for c_step in candidate_steps:
        # Use c_step to do rounding as well.
        lower_r = round_dec_down(lower_d / c_step) * c_step
        upper_r = round_dec_up(upper_d / c_step) * c_step
        num_steps = int(round_dec((upper_r - lower_r) / c_step))
        # If we are less than max_items, go with this.  (Note that smaller steps
        # are tried first).
        if num_steps <= max_items:
            ranges = []
            for i in xrange(num_steps):
                lower_i = input_type(lower_r + c_step * i)
                upper_i = input_type(lower_r + c_step * (i + 1))
                if i == num_steps - 1:
                    # make sure top item is rounded value
                    upper_i = input_type(upper_r)
                ranges.append((lower_i, upper_i))
            return ranges

    assert False, "Can't find a candidate set of ranges, logic error"



# URLs to make it easy to add more data for the test suite.

from django.conf.urls.defaults import *
from django.contrib import admin
admin.autodiscover()


urlpatterns = patterns('',
                       (r'^books/', 'django_easyfilters.tests.views.books'),
                       (r'^authors/', 'django_easyfilters.tests.views.authors'),
                       (r'^admin/', include(admin.site.urls)),

)




from datetime import date, timedelta
from dateutil.relativedelta import relativedelta
import math
import operator
import re

from django.core.exceptions import ValidationError
from django.db import models
from django.utils import formats
from django.utils.dates import MONTHS
from django.utils.text import capfirst
from django_easyfilters.queries import date_aggregation, value_counts, numeric_range_counts
from django_easyfilters.ranges import auto_ranges

try:
    from collections import namedtuple
    FilterChoice = namedtuple('FilterChoice', 'label count params link_type')
except ImportError:
    # We don't use it as a tuple, so this will do:
    class FilterChoice(object):
        def __init__(self, label, count, params, link_type):
            self.label, self.count, self.params, self.link_type = label, count, params, link_type


FILTER_ADD = 'add'
FILTER_REMOVE = 'remove'
FILTER_DISPLAY = 'display'


class Filter(object):
    """
    A Filter creates links/URLs that correspond to some DB filtering,
    and can apply the information from a URL to filter a QuerySet.
    """

    ### Public interface ###

    def __init__(self, field, model, params, query_param=None, order_by_count=False):
        self.field = field
        self.model = model
        self.params = params
        if query_param is None:
            query_param = field
        self.query_param = query_param
        self.order_by_count = order_by_count
        self.field_obj = self.model._meta.get_field(self.field)

        if self.field_obj.rel is not None:
            self.rel_model = self.field_obj.rel.to
            self.rel_field = self.field_obj.rel.get_related_field()
        # Make chosen an immutable sequence, to stop accidental mutation.
        self.chosen = tuple(self.choices_from_params())

    def apply_filter(self, qs):
        """
        Apply the filtering defined in params (request.GET) to the queryset qs,
        returning the new QuerySet.
        """
        chosen = list(self.chosen)
        while len(chosen) > 0:
            lookup = self.lookup_from_choice(chosen.pop())
            qs = qs.filter(**lookup)
        return qs

    def get_choices(self, qs):
        """
        Returns a list of namedtuples containing (label (as a string), count,
        params, link type)
        """
        raise NotImplementedError()

    ### Methods that are used by base implementation above ###

    def choices_from_params(self):
        out = []
        for p in self.params.getlist(self.query_param):
            try:
                choice = self.choice_from_param(p)
                out.append(choice)
            except ValueError:
                pass
        return out

    def choice_from_param(self, param):
        """
        Returns a native Python object representing something that has been
        chosen for a filter, converted from the string value in param.
        """
        try:
            return self.field_obj.to_python(param)
        except ValidationError:
            raise ValueError()

    def lookup_from_choice(self, choice):
        """
        Converts a choice value to a lookup dictionary that can be passed to
        QuerySet.filter() to do the filtering for that choice.
        """
        return {self.field: choice}

    ### Utility methods needed by most/all subclasses ###

    def param_from_choices(self, choices):
        """
        For a list of choices, return the parameter list that should be created.
        """
        return map(unicode, choices)

    def build_params(self, add=None, remove=None):
        """
        Builds a new parameter MultiDict.
        add is an optional item to add,
        remove is an option list of items to remove.
        """
        params = self.params.copy()
        chosen = list(self.chosen)
        if remove is not None:
            for r in remove:
                chosen.remove(r)
        else:
            if add not in chosen:
                chosen.append(add)
        if chosen:
            params.setlist(self.query_param, self.param_from_choices(chosen))
        else:
            del params[self.query_param]
        params.pop('page', None) # links should reset paging
        return params

    def sort_choices(self, qs, choices):
        """
        Sorts the choices by applying order_by_count if applicable.

        See also sort_choices_custom.
        """
        if self.order_by_count:
            choices.sort(key=operator.attrgetter('count'), reverse=True)
        return choices

    def normalize_add_choices(self, choices):
        return choices

    def get_choices_remove(self, qs):
        chosen = self.chosen
        choices = []
        for choice in chosen:
            display = self.display_choice(choice)
            if display is not None:
                choices.append(FilterChoice(display,
                                            None, # Don't need count for removing
                                            self.build_params(remove=[choice]),
                                            FILTER_REMOVE))
        return choices


class SingleValueMixin(object):
    """
    A mixin for filters where the field conceptually has just one value.
    """
    def normalize_add_choices(self, choices):
        addchoices = [(i, choice) for i, choice in enumerate(choices)
                      if choice.link_type == FILTER_ADD]
        if len(addchoices) == 1 and not self.field_obj.null:
            # No point giving people a choice of one, since all the results will
            # already have the selected value (apart from nullable fields, which
            # might have null)
            for i, c in addchoices:
                choices[i] = FilterChoice(label=choices[i].label,
                                          count=choices[i].count,
                                          link_type=FILTER_DISPLAY,
                                          params=None)
        return choices


class ChooseOnceMixin(SingleValueMixin):
    """
    A mixin for filters where you can only choose the filter once, and then
    remove the filter.
    """
    def get_choices(self, qs):
        choices_remove = self.get_choices_remove(qs)
        if len(choices_remove) > 0:
            return choices_remove
        else:
            choices_add = self.normalize_add_choices(self.get_choices_add(qs))
            return self.sort_choices(qs, choices_add)

    def get_choices_add(self, qs):
        raise NotImplementedError()


class ChooseAgainMixin(object):
    """
    A mixin for filters where it is possible to choose the filter more than
    once.
    """
    # This includes drill down, as well as many-valued fields.
    def get_choices(self, qs):
        # In general, can filter multiple times, so we can have multiple remove
        # links, and multiple add links, at the same time.
        choices_remove = self.get_choices_remove(qs)
        choices_add = self.normalize_add_choices(self.get_choices_add(qs))
        choices_add = self.sort_choices(qs, choices_add)
        return choices_remove + choices_add


class RelatedObjectMixin(object):
    """
    Mixin for fields that need to validate params against related field.
    """
    def choice_from_param(self, param):
        try:
            return self.rel_field.to_python(param)
        except ValidationError:
            raise ValueError()


class SimpleQueryMixin(object):
    """
    Mixin for filters that do a simple DB query on main table to get counts.
    """
    def get_values_counts(self, qs):
        """
        Returns a SortedDict dictionary of {value: count}.

        The order is the underlying order produced by sorting ascending on the
        DB field.
        """
        return value_counts(qs, self.field)


class RangeFilterMixin(ChooseAgainMixin, SingleValueMixin):

    # choice_type must be set to a class that provides the static method
    # 'from_param' and instance methods 'make_lookup' and 'display', and the
    # __cmp__ and __eq__ methods for sorting.
    choice_type = None

    def choice_from_param(self, param):
        return self.choice_type.from_param(param)

    def choices_from_params(self):
        choices = super(RangeFilterMixin, self).choices_from_params()
        choices.sort()
        return choices

    def lookup_from_choice(self, choice):
        return choice.make_lookup(self.field)

    def display_choice(self, choice):
        return choice.display()

    def get_choices_remove(self, qs):
        # Due to drill down, if a broader param is removed, the more specific
        # params must be removed too. We assume we can do an ordering on
        # whatever 'choice' objects are in chosen, and 'greater' means 'more
        # specific'.
        chosen = list(self.chosen)
        out = []
        for i, choice in enumerate(chosen):
            to_remove = [c for c in chosen if c >= choice]
            out.append(FilterChoice(self.display_choice(choice),
                                    None,
                                    self.build_params(remove=to_remove),
                                    FILTER_REMOVE))
        return out


### Concrete filter classes that are used by FilterSet ###

class ValuesFilter(ChooseOnceMixin, SimpleQueryMixin, Filter):
    """
    Fallback Filter for various kinds of simple values.
    """
    def display_choice(self, choice):
        retval = unicode(choice)
        if retval == u'':
            return u'(empty)'
        else:
            return retval

    def get_choices_add(self, qs):
        """
        Called by 'get_choices', this is usually the one to override.
        """
        count_dict = self.get_values_counts(qs)
        return [FilterChoice(self.display_choice(val),
                             count,
                             self.build_params(add=val),
                             FILTER_ADD)
                for val, count in count_dict.items()]


class ChoicesFilter(ValuesFilter):
    """
    Filter for fields that have 'choices' defined.
    """
    # Need to do the following:
    # 1) ensure we only display options that are in 'choices'
    # 2) ensure the order is the same as in choices
    # 3) make display value = the second element in choices' tuples.
    def __init__(self, *args, **kwargs):
        super(ChoicesFilter, self).__init__(*args, **kwargs)
        self.choices_dict = dict(self.field_obj.flatchoices)

    def display_choice(self, choice):
        # 3) above
        return self.choices_dict.get(choice, choice)

    def get_choices_add(self, qs):
        count_dict = self.get_values_counts(qs)
        choices = []
        for val, display in self.field_obj.choices:
            # 1), 2) above
            if val in count_dict:
                # We could use the value 'display' here, but for consistency
                # call display_choice() in case it is overriden.
                choices.append(FilterChoice(self.display_choice(val),
                                            count_dict[val],
                                            self.build_params(add=val),
                                            FILTER_ADD))
        return choices


class ForeignKeyFilter(ChooseOnceMixin, SimpleQueryMixin, RelatedObjectMixin, Filter):
    """
    Filter for ForeignKey fields.
    """
    def display_choice(self, choice):

        lookup = {self.rel_field.name: choice}
        try:
            obj = self.rel_model.objects.get(**lookup)
        except self.rel_model.DoesNotExist:
            return None
        return unicode(obj)

    def get_choices_add(self, qs):
        count_dict = self.get_values_counts(qs)
        lookup = {self.rel_field.name + '__in': count_dict.keys()}
        objs = self.rel_model.objects.filter(**lookup)
        choices = []

        for o in objs:
            pk = getattr(o, self.rel_field.attname)
            choices.append(FilterChoice(unicode(o),
                                        count_dict[pk],
                                        self.build_params(add=pk),
                                        FILTER_ADD))
        return choices


class ManyToManyFilter(ChooseAgainMixin, RelatedObjectMixin, Filter):

    def get_values_counts(self, qs):
        # It is easiest to base queries around the intermediate table, in order
        # to get counts.
        through = self.field_obj.rel.through
        rel_model = self.rel_model

        assert rel_model != self.model, "Can't cope with this yet..."
        fkey_this = [f for f in through._meta.fields
                     if f.rel is not None and f.rel.to is self.model][0]
        fkey_other = [f for f in through._meta.fields
                      if f.rel is not None and f.rel.to is rel_model][0]

        # We need to limit items by what is in the main QuerySet (which might
        # already be filtered).
        m2m_objs = through.objects.filter(**{fkey_this.name + '__in':qs})

        # We need to exclude items in other table that we have already filtered
        # on, because they are not interesting.
        m2m_objs = m2m_objs.exclude(**{fkey_other.name + '__in': self.chosen})

        # Now get counts:
        field_name = fkey_other.name
        return value_counts(m2m_objs, field_name)

    def get_choices_add(self, qs):
        count_dict = self.get_values_counts(qs)
        # Now, need to lookup objects on related table, to display them.
        objs = self.rel_model.objects.filter(pk__in=count_dict.keys())

        return [FilterChoice(unicode(o),
                             count_dict[o.pk],
                             self.build_params(add=o.pk),
                             FILTER_ADD)
                for o in objs]

    def get_choices_remove(self, qs):
        chosen = self.chosen
        # Do a query in bulk to get objs corresponding to choices.
        objs = self.rel_model.objects.filter(pk__in=chosen)

        # We want to preserve order of items in params, so use the original
        # 'chosen' list, rather than objs.
        obj_dict = dict([(obj.pk, obj) for obj in objs])
        return [FilterChoice(unicode(obj_dict[choice]),
                             None, # Don't need count for removing
                             self.build_params(remove=[choice]),
                             FILTER_REMOVE)
                for choice in chosen if choice in obj_dict]


class DateRangeType(object):

    all = {} # Keep a cache, so that we have unique instances

    def __init__(self, level, single, label, regex):
        self.level, self.single, self.label = level, single, label
        self.regex = re.compile((r'^(%s)$' % regex) if single else
                                (r'^(%s)..(%s)$' % (regex, regex)))
        DateRangeType.all[(level, single)] = self

    def __repr__(self):
        return '<DateRange %d %s %s>' % (self.level,
                                         "single" if self.single else "multi",
                                         self.label)

    def __cmp__(self, other):
        if other is None:
            return 1
        else:
            return cmp((self.level, self.single),
                       (other.level, other.single))

    @classmethod
    def get(cls, level, single):
        return cls.all[(level, single)]

    @property
    def dateattr(self):
        # The attribute of a date object that we truncate to when collapsing results.
        return self.label

    @property
    def relativedeltaattr(self):
        # The attribute to use for calculations using relativedelta
        return self.label + 's'

    def drilldown(self):
        if self is DAY:
            return None
        if not self.single:
            return DateRangeType.get(self.level, True)
        else:
            # We always drill down to 'single', and then generate
            # ranges (i.e. multi) if appropriate.
            return DateRangeType.get(self.level + 1, True)


_y, _ym, _ymd = r'\d{4}', r'\d{4}-\d{2}', r'\d{4}-\d{2}-\d{2}'
YEARGROUP   = DateRangeType(1, False, 'year',  _y)
YEAR        = DateRangeType(1, True,  'year',  _y)
MONTHGROUP  = DateRangeType(2, False, 'month', _ym)
MONTH       = DateRangeType(2, True,  'month', _ym)
DAYGROUP    = DateRangeType(3, False, 'day',   _ymd)
DAY         = DateRangeType(3, True,  'day',   _ymd)


class DateChoice(object):
    """
    Represents a choice of date. Params are converted to this, and this is used
    to build new params and format links.

    It can represent a year, month or day choice, or a range (start, end, both
    inclusive) of any of these choice.
    """

    def __init__(self, range_type, values):
        self.range_type = range_type
        self.values = values

    def __unicode__(self):
        # This is called when converting to URL
        return '..'.join(self.values)

    def __repr__(self):
        return '<DateChoice %s %s>' % (self.range_type, self.__unicode__())

    def __cmp__(self, other):
        # 'greater' means more specific.
        return cmp((self.range_type, self.values),
                   (other.range_type, other.values))

    def display(self):
        # Called for user presentable string
        if self.range_type.single:
            value = self.values[0]
            parts = value.split('-')
            if self.range_type == YEAR:
                return parts[0]
            elif self.range_type == MONTH:
                return unicode(MONTHS[int(parts[1])])
            elif self.range_type == DAY:
                return str(int(parts[-1]))
        else:
            return u'-'.join([DateChoice(DateRangeType.get(self.range_type.level, True),
                                         [val]).display()
                              for val in self.values])

    @staticmethod
    def datetime_to_value(range_type, dt):
        if range_type == YEAR:
            return '%04d' % dt.year
        elif range_type == MONTH:
            return '%04d-%02d' % (dt.year, dt.month)
        else:
            return '%04d-%02d-%02d' % (dt.year, dt.month, dt.day)

    @staticmethod
    def from_datetime(range_type, dt):
        return DateChoice(range_type, [DateChoice.datetime_to_value(range_type, dt)])

    @staticmethod
    def from_datetime_range(range_type, dt1, dt2):
        return DateChoice(DateRangeType.get(range_type.level, False),
                          [DateChoice.datetime_to_value(range_type, dt1),
                           DateChoice.datetime_to_value(range_type, dt2)])

    @staticmethod
    def from_param(param):
        for drt in DateRangeType.all.values():
            m = drt.regex.match(param)
            if m is not None:
                return DateChoice(drt, list(m.groups()))
        raise ValueError()

    def make_lookup(self, field_name):
        # It's easier to do this all using datetime comparisons than have a
        # separate path for the single year/month/day case.
        if self.range_type.single:
            start, end = self.values[0], self.values[0]
        else:
            start, end = self.values

        start_parts = map(int, start.split('-'))
        end_parts = map(int, end.split('-'))

        # Fill the parts we don't have with '1' so that e.g. 2000 becomes
        # 2000-1-1
        start_parts = start_parts + [1] * (3 - len(start_parts))
        end_parts = end_parts + [1] * (3 - len(end_parts))
        start_date = date(start_parts[0], start_parts[1], start_parts[2])
        end_date = date(end_parts[0], end_parts[1], end_parts[2])

        # Now add one year/month/day:
        end_date = end_date + relativedelta(**{self.range_type.relativedeltaattr: 1})

        return {field_name + '__gte': start_date,
                field_name + '__lt':  end_date}


class DateTimeFilter(RangeFilterMixin, Filter):

    choice_type = DateChoice

    max_depth_levels = {'year': YEAR.level,
                        'month': MONTH.level,
                        None: DAY.level + 1}

    def __init__(self, *args, **kwargs):
        self.max_links = kwargs.pop('max_links', 12)
        self.max_depth = kwargs.pop('max_depth', None)
        assert self.max_depth in ['year', 'month', None]
        self.max_depth_level = self.max_depth_levels[self.max_depth]
        super(DateTimeFilter, self).__init__(*args, **kwargs)

    def get_choices_add(self, qs):
        chosen = list(self.chosen)
        range_type = None

        if len(chosen) > 0:
            range_type = chosen[-1].range_type.drilldown()
            if range_type is None:
                return []

        if range_type is None:
            # Get some initial idea of range
            date_range = qs.aggregate(first=models.Min(self.field),
                                      last=models.Max(self.field))
            first = date_range['first']
            last = date_range['last']
            if first is None or last is None:
                # No values, can't drill down:
                return []
            if first.year == last.year:
                if first.month == last.month:
                    range_type = DAY
                else:
                    range_type = MONTH
            else:
                range_type = YEAR

        date_qs = qs.dates(self.field, range_type.label)
        results = date_aggregation(date_qs)

        date_choice_counts = self.collapse_results(results, range_type)

        choices = []
        # Additional display links, to give context for choices if necessary.
        if len(date_choice_counts) > 0:
            choices.extend(self.bridge_choices(chosen, date_choice_counts))

        for date_choice, count in date_choice_counts:
            if date_choice in chosen:
                continue

            # To ensure we get the bridge choices, which are useful, we check
            # self.max_depth_level late on and bailout here.
            if range_type.level > self.max_depth_level:
                continue

            choices.append(FilterChoice(date_choice.display(),
                                        count,
                                        self.build_params(add=date_choice),
                                        FILTER_ADD))
        return choices

    def collapse_results(self, results, range_type):
        if len(results) > self.max_links:
            # If range_type is month/day, we don't want any possibility of the
            # buckets wrapping over to the next year/month, so we set first and
            # last accordingly
            if range_type == MONTH:
                first, last = 1, 12
            elif range_type == DAY:
                first, last = 1, 31
            else:
                first = results[0][0].year
                last = results[-1][0].year

            # We need to split into even sized buckets, so it looks nice.
            span =  last - first + 1
            bucketsize = int(math.ceil(float(span) / self.max_links))
            numbuckets = int(math.ceil(float(span) / bucketsize))

            buckets = [[] for i in range(numbuckets)]
            for row in results:
                val = getattr(row[0], range_type.dateattr)
                bucketnum = int(math.floor(float(val - first)/bucketsize))
                buckets[bucketnum].append(row)

            dt_template = results[0][0]
            date_choice_counts = []
            for i, bucket in enumerate(buckets):
                count = sum(row[1] for row in bucket)
                start_val = first + bucketsize * i
                start_date = dt_template.replace(**dict({range_type.dateattr: start_val}))
                end_date = start_date + relativedelta(**dict({range_type.relativedeltaattr: bucketsize - 1}))

                choice = DateChoice.from_datetime_range(range_type, start_date, end_date)
                date_choice_counts.append((choice, count))
        else:
            date_choice_counts = [(DateChoice.from_datetime(range_type, dt), count)
                                  for dt, count in results]
        return date_choice_counts

    def bridge_choices(self, chosen, choices):
        # Returns FILTER_DISPLAY type choices to bridge from what is chosen
        # (which might be nothing) to the first 'add' link, to give context to
        # the link.
        if len(choices) == 0:
            return []
        if len(chosen) == 0:
            chosen_level = 0
        else:
            chosen_level = chosen[-1].range_type.level

        # first choice in list can act as template, as it will have all the
        # values we need.
        new_choice = choices[0][0]
        new_level = new_choice.range_type.level

        retval = []
        while chosen_level < new_level - 1:
            chosen_level += 1
            if chosen_level > self.max_depth_level:
                continue
            date_choice = DateChoice(DateRangeType.get(chosen_level, True),
                                     new_choice.values)
            retval.append(FilterChoice(date_choice.display(),
                                       None, None,
                                       FILTER_DISPLAY))
        return retval


class RangeEnd(object):
    """
    Simple structure to store part of a range
    """
    def __init__(self, value, inclusive):
        # value is some generic value, inclusive is a bool specifying where this
        # value is included as part of the range.
        self.value, self.inclusive = value, inclusive


def make_numeric_range_choice(to_python, to_str):
    """
    Returns a Choice class that represents a numeric choice range,
    using the passed in 'to_python' and 'to_str' callables to do
    conversion to/from native data types.
    """

    class NumericRangeChoice(object):
        def __init__(self, values):
            # Values are instances of RangeEnd
            self.values = values

        def display(self):
            return '-'.join([str(v.value) for v in self.values])

        @classmethod
        def from_param(cls, param):
            vals = []
            for p in param.split('..', 1):
                inclusive = False
                if p.endswith('i'):
                    inclusive = True
                    p = p[:-1]

                try:
                    val = to_python(p)
                    vals.append(RangeEnd(val, inclusive))
                except ValidationError:
                    raise ValueError()
            return cls(vals)

        def make_lookup(self, field_name):
            if len(self.values) == 1:
                return {field_name: self.values[0].value}
            else:
                start, end = self.values[0], self.values[1]
                return {field_name + '__gt' + ('e' if start.inclusive else ''): start.value,
                        field_name + '__lt' + ('e' if end.inclusive else ''): end.value}

        def __unicode__(self):
            return '..'.join([to_str(v.value) + ('i' if v.inclusive else '')
                              for v in self.values])

        def __repr__(self):
            return '<NumericChoice %s>' % self.__unicode__()

        def __cmp__(self, other):
            # 'greater' means more specific.
            if other is None:
                return cmp(self.values, None)
            else:
                if len(self.values) != len(other.values):
                    # one value is more specific than two
                    return -cmp(len(self.values), len(other.values))
                elif len(self.values) == 1:
                    return 0
                else:
                    # Larger difference means less specific
                    return -cmp(self.values[1].value - self.values[0].value,
                                other.values[1].value - other.values[0].value)

    return NumericRangeChoice


class NumericRangeFilter(RangeFilterMixin, Filter):

    def __init__(self, field, model, params, **kwargs):
        self.max_links = kwargs.pop('max_links', 5)
        self.drilldown = kwargs.pop('drilldown', True)
        self.ranges = kwargs.pop('ranges', None)
        field_obj = model._meta.get_field(field)
        self.choice_type = make_numeric_range_choice(field_obj.to_python, str)
        super(NumericRangeFilter, self).__init__(field, model, params, **kwargs)

    def display_choice(self, c):
        if self.ranges is None:
            return c.display()
        if len(c.values) == 1:
            return c.display()
        for r in self.ranges:
            if r[0] == c.values[0].value and r[1] == c.values[1].value:
                # found a match in ranges
                if len(r) > 2:
                    # found a label, so use it
                    return r[2]
                else:
                    return c.display()
        return c.display()

    def get_choices_add(self, qs):
        chosen = list(self.chosen)
        range_type = None

        if not self.drilldown and len(chosen) > 0:
            return []

        all_vals = qs.values_list(self.field).distinct()

        num = all_vals.count()

        choices = []
        if num <= self.max_links:
            val_counts = value_counts(qs, self.field)
            for v, count in val_counts.items():
                choice = self.choice_type([RangeEnd(v, True)])
                choices.append(FilterChoice(self.display_choice(choice),
                                            count,
                                            self.build_params(add=choice),
                                            FILTER_ADD))
        else:
            if self.ranges is None:
                val_range = qs.aggregate(lower=models.Min(self.field),
                                         upper=models.Max(self.field))
                ranges = auto_ranges(val_range['lower'], val_range['upper'], self.max_links)
            else:
                ranges = self.ranges

            val_counts = numeric_range_counts(qs, self.field, ranges)
            for i, (vals, count) in enumerate(val_counts.items()):
                # For the lower bound, we make it inclusive only if it the first
                # choice. The upper bound is always inclusive. This gives
                # filters that behave sensibly e.g. with 10-20, 20-30, 30-40,
                # the first will include 10 and 20, the second will exlude 20.
                choice = self.choice_type([RangeEnd(vals[0], i == 0),
                                           RangeEnd(vals[1], True)])
                choices.append(FilterChoice(self.display_choice(choice),
                                            count,
                                            self.build_params(add=choice),
                                            FILTER_ADD))

        return choices



from django.db import models
from django.db.backends.util import typecast_timestamp
from django.db.models.sql.compiler import SQLCompiler
from django.db.models.sql.constants import MULTI
from django.db.models.sql.datastructures import Date
from django.db.models.sql.subqueries import AggregateQuery
from django.utils.datastructures import SortedDict


# Some fairly brittle, low level stuff, to get the aggregation
# queries we need.


class DateAggregateQuery(AggregateQuery):
    # Need to override to return a compiler not in django.db.models.sql.compiler
    def get_compiler(self, using=None, connection=None):
        return DateAggregateCompiler(self, connection, using)

    def get_counts(self, using):
        from django.db import connections
        connection = connections[using]
        return list(self.get_compiler(using, connection).results_iter())


class DateAggregateCompiler(SQLCompiler):
    def results_iter(self):
        needs_string_cast = self.connection.features.needs_datetime_string_cast

        for rows in self.execute_sql(MULTI):
            for row in rows:
                if needs_string_cast:
                    vals = [typecast_timestamp(str(row[0])),
                            row[1]]
                else:
                    vals = row
                yield vals

    def as_sql(self, qn=None):
        sql = ('SELECT %s, COUNT(%s) FROM (%s) subquery GROUP BY (%s) ORDER BY (%s)' % (
                DateWithAlias.alias, DateWithAlias.alias, self.query.subquery,
                DateWithAlias.alias, DateWithAlias.alias)
               )
        params = self.query.sub_params
        return (sql, params)


class DateWithAlias(Date):
    alias = 'easyfilter_date_alias'
    def as_sql(self, qn, connection):
        return super(DateWithAlias, self).as_sql(qn, connection) + ' as ' + self.alias


def date_aggregation(date_qs):
    """
    Performs an aggregation for a supplied DateQuerySet
    """
    # The DateQuerySet gives us a query that we need to clone and hack
    date_q = date_qs.query.clone()
    date_q.distinct = False

    # Replace 'select' to add an alias
    date_obj = date_q.select[0]
    date_q.select = [DateWithAlias(date_obj.col, date_obj.lookup_type)]

    # Now use as a subquery to do aggregation
    query = DateAggregateQuery(date_qs.model)
    query.add_subquery(date_q, date_qs.db)
    return query.get_counts(date_qs.db)


def value_counts(qs, fieldname):
    """
    Performs a simple query returning the count of each value of
    the field 'fieldname' in the QuerySet, returning the results
    as a SortedDict of value: count
    """
    values_counts = qs.values_list(fieldname).order_by(fieldname).annotate(models.Count(fieldname))
    count_dict = SortedDict()
    for val, count in values_counts:
        count_dict[val] = count
    return count_dict


class NumericAggregateQuery(AggregateQuery):
    # Need to override to return a compiler not in django.db.models.sql.compiler
    def get_compiler(self, using=None, connection=None):
        return  NumericAggregateCompiler(self, connection, using)

    def get_counts(self, using):
        from django.db import connections
        connection = connections[using]
        return list(self.get_compiler(using, connection).results_iter())


class NumericAggregateCompiler(SQLCompiler):
    def results_iter(self):
        for rows in self.execute_sql(MULTI):
            for row in rows:
                yield row

    def as_sql(self, qn=None):
        sql = ('SELECT %s, COUNT(%s) FROM (%s) subquery GROUP BY (%s) ORDER BY (%s)' % (
                NumericValueRange.alias, NumericValueRange.alias, self.query.subquery,
                NumericValueRange.alias, NumericValueRange.alias)
               )
        params = self.query.sub_params
        return (sql, params)


class NumericValueRange(object):
    alias = 'easyfilter_number_range_alias'
    def __init__(self, col, ranges):
        # ranges is list of (lower, upper) bounds we want to find, where 'lower'
        # is inclusive and upper is exclusive.
        self.col = col
        self.ranges = ranges

    # TODO - do we need 'relabel_aliases', like 'Date'?

    def as_sql(self, qn, connection):
        if isinstance(self.col, (list, tuple)):
            col = '%s.%s' % tuple([qn(c) for c in self.col])
        else:
            col = self.col

        # Build up case expression.
        clause = (['CASE '] +
                  ['WHEN %s > %s AND %s <= %s THEN %s ' % (col, val[0], col, val[1], i)
                   for i, val in enumerate(self.ranges)] +
                  # An inclusive lower limit for the first item in ranges:
                  ['WHEN %s = %s THEN 0 ' % (col, self.ranges[0][0])] +
                  ['ELSE %s END ' % len(self.ranges)] +
                  ['as %s' % self.alias])
        return ''.join(clause)


def numeric_range_counts(qs, fieldname, ranges):

    # Build the query:
    query = qs.values_list(fieldname).query.clone()
    query.select[0] = NumericValueRange(query.select[0], ranges)

    agg_query = NumericAggregateQuery(qs.model)
    agg_query.add_subquery(query, qs.db)
    results = agg_query.get_counts(qs.db)

    count_dict = SortedDict()
    for val, count in results:
        try:
            r = ranges[val]
        except IndexError:
            # Include in the top range - this could be a rounding error
            r = ranges[-1]
        count_dict[r] = count
    return count_dict




from django import template
from django.utils.safestring import mark_safe
from django.utils.html import escape
from django.utils.text import capfirst

from django_easyfilters.filters import FILTER_ADD, FILTER_REMOVE, FILTER_DISPLAY, \
    ValuesFilter, ChoicesFilter, ForeignKeyFilter, ManyToManyFilter, DateTimeFilter, NumericRangeFilter


def non_breaking_spaces(val):
    # This helps a lot with presentation, by stopping the links+count from being
    # split over a line end.
    val = val.replace(u'-', u'\u2011')
    return mark_safe(u'&nbsp;'.join(escape(part) for part in val.split(u' ')))


class FilterSet(object):

    template = """
<div class="filterline"><span class="filterlabel">{{ filterlabel }}:</span>
{% for choice in choices %}
  {% if choice.link_type == 'add' %}
    <span class="addfilter"><a href="{{ choice.url }}" title="Add filter">{{ choice.label }}&nbsp;({{ choice.count }})</a></span>&nbsp;&nbsp;
  {% else %}
    {% if choice.link_type == 'remove' %}
    <span class="removefilter"><a href="{{ choice.url }}" title="Remove filter">{{ choice.label }}&nbsp;&laquo;&nbsp;</a></span>
    {% else %}
      <span class="displayfilter">{{ choice.label }}</span>
    {% endif %}
  {% endif %}
{% endfor %}
</div>
"""

    def __init__(self, queryset, params):
        self.params = params
        self.model = queryset.model
        self.filters = self.setup_filters()
        self.qs = self.apply_filters(queryset)

    def apply_filters(self, queryset):
        for f in self.filters:
            queryset = f.apply_filter(queryset)
        return queryset

    def render_filter(self, filter_, qs):
        field_obj = self.model._meta.get_field(filter_.field)
        choices = filter_.get_choices(qs)
        ctx = {'filterlabel': capfirst(field_obj.verbose_name)}
        ctx['choices'] = [dict(label=non_breaking_spaces(c.label),
                               url=u'?' + c.params.urlencode() \
                                   if c.link_type != FILTER_DISPLAY else None,
                               link_type=c.link_type,
                               count=c.count)
                          for c in choices]
        return self.get_template(filter_.field).render(template.Context(ctx))

    def get_template(self, field_name):
        return template.Template(self.template)

    def render(self):
        return mark_safe(u'\n'.join(self.render_filter(f, self.qs) for f in self.filters))

    def get_fields(self):
        return self.fields

    def get_filter_for_field(self, field):
        f, model, direct, m2m = self.model._meta.get_field_by_name(field)
        if f.rel is not None:
            if m2m:
                return ManyToManyFilter
            else:
                return ForeignKeyFilter
        elif f.choices:
            return ChoicesFilter
        else:
            type_ = f.get_internal_type()
            if type_ == 'DateField' or type_ == 'DateTimeField':
                return DateTimeFilter
            elif type_ == 'DecimalField':
                return NumericRangeFilter
            else:
                return ValuesFilter

    def setup_filters(self):
        filters = []
        for i, f in enumerate(self.get_fields()):
            klass = None
            if isinstance(f, basestring):
                opts = {}
                field_name = f
            else:
                opts = f[1]
                field_name = f[0]
                if len(f) > 2:
                    klass = f[2]
            if klass is None:
                klass = self.get_filter_for_field(field_name)
            filters.append(klass(field_name, self.model, self.params, **opts))
        return filters

    def __unicode__(self):
        return self.render()



from django.db import models


BINDING_CHOICES = [
    ('H', 'Hardback'),
    ('P', 'Paperback'),
    ('C', 'Cloth'),
]

class Author(models.Model):
    name = models.CharField(max_length=50)
    likes = models.IntegerField(default=0)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ['name']

class Genre(models.Model):
    name = models.CharField(max_length=50)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ['name']

class Book(models.Model):
    name = models.CharField(max_length=100)
    binding = models.CharField(max_length=2, choices=BINDING_CHOICES)
    authors = models.ManyToManyField(Author)
    genre = models.ForeignKey(Genre)
    price = models.DecimalField(max_digits=6, decimal_places=2)
    date_published = models.DateField()
    edition = models.IntegerField(default=1)

    def __unicode__(self):
        return self.name




# admin registration to make it easy to add more data for the test suite.

from models import *
from django.contrib import admin

class BookAdmin(admin.ModelAdmin):
    def authors(obj):
        return ", ".join(unicode(a) for a in obj.authors.all())
    list_display = ["name", authors, "binding", "genre", "price", "date_published", "edition"]
    list_editable = ["binding", "genre", "price", "date_published", "edition"]
    list_filter = ["genre", "authors", "binding", "price"]


class AuthorAdmin(admin.ModelAdmin):
    list_display = ["id", "name", "likes"]
    list_editable = ["name", "likes"]

admin.site.register(Book, BookAdmin)
admin.site.register(Author, AuthorAdmin)
admin.site.register(Genre)



# -*- coding: utf-8; -*-

from datetime import datetime, date
from decimal import Decimal
import operator

from django.http import QueryDict
from django.test import TestCase
from django.utils.datastructures import MultiValueDict

from django_easyfilters.filterset import FilterSet
from django_easyfilters.filters import \
    FILTER_ADD, FILTER_REMOVE, FILTER_DISPLAY, \
    ForeignKeyFilter, ValuesFilter, ChoicesFilter, ManyToManyFilter, DateTimeFilter, NumericRangeFilter

from models import Book, Genre, Author, BINDING_CHOICES


class TestFilterSet(TestCase):

    # Tests are written so that adding new data to fixtures won't break the
    # tests, so numbers/values are compared using DB queries. Extra care is
    # taken to ensure that there is some data that matches what we are assuming
    # is there.
    fixtures = ['django_easyfilters_tests']

    def test_queryset_no_filters(self):
        class BookFilterSet(FilterSet):
            fields = []

        qs = Book.objects.all()
        data = QueryDict('')
        f = BookFilterSet(qs, data)
        self.assertEqual(qs.count(), f.qs.count())

    def test_filterset_render(self):
        """
        Smoke test to ensure that filtersets can be rendered
        """
        class BookFilterSet(FilterSet):
            fields = [
                'genre',
                ]

        qs = Book.objects.all()
        fs = BookFilterSet(qs, QueryDict(''))
        rendered = fs.render()
        self.assertTrue('Genre' in rendered)
        self.assertEqual(rendered, unicode(fs))

        # And when in 'already filtered' mode:
        choice = fs.filters[0].get_choices(qs)[0]
        fs_filtered = BookFilterSet(qs, choice.params)
        rendered_2 = fs_filtered.render()
        self.assertTrue('Genre' in rendered_2)

    def test_get_filter_for_field(self):
        """
        Ensures that the get_filter_for_field method chooses appropriately.
        """
        class BookFilterSet(FilterSet):
            fields = [
                'genre',
                'edition',
                'binding',
                'authors',
                'date_published',
                'price',
                ]

        fs = BookFilterSet(Book.objects.all(), QueryDict(''))
        self.assertEqual(ForeignKeyFilter, type(fs.filters[0]))
        self.assertEqual(ValuesFilter, type(fs.filters[1]))
        self.assertEqual(ChoicesFilter, type(fs.filters[2]))
        self.assertEqual(ManyToManyFilter, type(fs.filters[3]))
        self.assertEqual(DateTimeFilter, type(fs.filters[4]))
        self.assertEqual(NumericRangeFilter, type(fs.filters[5]))

    def test_specify_custom_filter(self):
        class AuthorFilterSet(FilterSet):
            fields = [
                ('likes', {}, NumericRangeFilter)
                ]

        fs = AuthorFilterSet(Author.objects.all(), QueryDict(''))
        self.assertEqual(NumericRangeFilter, type(fs.filters[0]))


class TestFilters(TestCase):
    fixtures = ['django_easyfilters_tests']

    def do_invalid_query_param_test(self, make_filter, params):
        """
        Utility to test filters with invalid query parameters.

        make_filter should a callable that accepts MultiValueDict
        and returns a filter.
        """
        f = make_filter(params)
        f_empty = make_filter(MultiValueDict())
        qs = f.model.objects.all()

        # invalid param should be ignored
        qs_filtered = f.apply_filter(qs)
        self.assertEqual(list(qs_filtered),
                         list(qs))

        self.assertEqual(list(f.get_choices(qs)),
                         list(f_empty.get_choices(qs)))

    def do_missing_related_object_test(self, make_filter, params):
        """
        Utility to test filters with query strings representing objects
        not in the database.

        make_filter should a callable that accepts MultiValueDict
        and returns a filter.
        """
        f = make_filter(params)
        qs = f.model.objects.all()

        # choices should render without error, and with no
        # 'remove' links.
        qs_filtered = f.apply_filter(qs)
        choices = list(f.get_choices(qs))
        self.assertFalse(any(c.link_type == FILTER_REMOVE
                             for c in choices))

    def test_foreignkey_filters_produced(self):
        """
        A ForeignKey should produce a list of the possible related objects,
        with counts.
        """
        # Make another Genre that isn't used
        new_g, created = Genre.objects.get_or_create(name='Nonsense')
        assert created

        data = MultiValueDict()
        filter_ = ForeignKeyFilter('genre', Book, data)
        qs = Book.objects.all()

        choices = [(c.label, c.count) for c in filter_.get_choices(qs)]

        reached = [False, False]
        for g in Genre.objects.all():
            count = g.book_set.count()
            if count == 0:
                reached[0] = True
                self.assertTrue((g.name, count) not in choices)
            else:
                reached[1] = True
                self.assertTrue((g.name, count) in choices)

        self.assertTrue(reached[0])
        self.assertTrue(reached[1])

    def test_foreignkey_params_produced(self):
        """
        A ForeignKey filter shoud produce params that cause the query to be
        limited by that filter.
        """
        qs = Book.objects.all()
        data = MultiValueDict()
        filter1 = ForeignKeyFilter('genre', Book, data)
        choices = filter1.get_choices(qs)

        # If we use the params from e.g. the first choice, that should produce a
        # filtered qs when fed back in (i.e. when we 'click' on that option we
        # should get a filter on it).
        reached = False
        for choice in choices:
            reached = True
            filter2 = ForeignKeyFilter('genre', Book, choice.params)
            qs_filtered = filter2.apply_filter(qs)
            self.assertEqual(len(qs_filtered), choice.count)
            for book in qs_filtered:
                self.assertEqual(unicode(book.genre), choice.label)
        self.assertTrue(reached)

    def test_foreignkey_remove_link(self):
        """
        Ensure that a ForeignKey Filter will turn into a 'remove' link when an
        item has been selected.
        """
        qs = Book.objects.all()
        data = MultiValueDict()
        filter1 = ForeignKeyFilter('genre', Book, data)
        choices = filter1.get_choices(qs)
        choice = choices[0]

        filter2 = ForeignKeyFilter('genre', Book, choice.params)
        qs_filtered = filter2.apply_filter(qs)
        choices2 = filter2.get_choices(qs_filtered)

        # Should have one item
        self.assertEqual(1, len(choices2))
        self.assertEqual(choices2[0].link_type, FILTER_REMOVE)
        self.assertEqual(choices2[0].label, choice.label)

        # 'Clicking' should remove filtering
        filter3 = ForeignKeyFilter('genre', Book, choices2[0].params)
        qs_reverted = filter3.apply_filter(qs)
        self.assertEqual(qs, qs_reverted)

    def test_foreignkey_invalid_query(self):
        self.do_invalid_query_param_test(lambda params:
                                             ForeignKeyFilter('genre', Book, params),
                                         MultiValueDict({'genre':['xxx']}))
        self.do_missing_related_object_test(lambda params:
                                                ForeignKeyFilter('genre', Book, params),
                                            MultiValueDict({'genre':['1000']}))

    def test_values_filter(self):
        """
        Tests for ValuesFilter
        """
        # We combine the tests for brevity
        filter1 = ValuesFilter('edition', Book, MultiValueDict())
        qs = Book.objects.all()
        choices = filter1.get_choices(qs)

        for choice in choices:
            count = Book.objects.filter(edition=choice.params.values()[0]).count()
            self.assertEqual(choice.count, count)

            # Check the filtering
            filter2 = ValuesFilter('edition', Book, choice.params)
            qs_filtered = filter2.apply_filter(qs)
            self.assertEqual(len(qs_filtered), choice.count)
            for book in qs_filtered:
                self.assertEqual(unicode(book.edition), choice.label)

            # Check we've got a 'remove link' on filtered.
            choices_filtered = filter2.get_choices(qs)
            self.assertEqual(1, len(choices_filtered))
            self.assertEqual(choices_filtered[0].link_type, FILTER_REMOVE)


        # Check list is full, and in right order
        self.assertEqual([unicode(v) for v in Book.objects.values_list('edition', flat=True).order_by('edition').distinct()],
                         [choice.label for choice in choices])

    def test_choices_filter(self):
        """
        Tests for ChoicesFilter
        """
        filter1 = ChoicesFilter('binding', Book, MultiValueDict())
        qs = Book.objects.all()
        choices = filter1.get_choices(qs)
        # Check:
        # - order is correct.
        # - all values present (guaranteed by fixture data)
        # - choice display value is used.

        binding_choices_db = [b[0] for b in BINDING_CHOICES]
        binding_choices_display = [b[1] for b in BINDING_CHOICES]
        self.assertEqual([c.label for c in choices], binding_choices_display)

        # Check choice db value in params
        for c in choices:
            self.assertTrue(c.params.values()[0] in binding_choices_db)

    def test_normalize_choices(self):
        # We shouldn't get links for non-nullable fields when there is only one choice.

        # Make sure there are no other books in 1975
        clark = Author.objects.get(name='Arthur C. Clarke')
        Book.objects.filter(date_published__year=1975).exclude(authors=clark.id).delete()
        qs = Book.objects.filter(date_published__year=1975)
        self.assertEqual(len(qs), 1)

        filter1 = ChoicesFilter('binding', Book, MultiValueDict())
        choices1 = filter1.get_choices(qs)
        self.assertEqual(len(choices1), 1)
        self.assertEqual(choices1[0].link_type, FILTER_DISPLAY)

        filter2 = ForeignKeyFilter('genre', Book, MultiValueDict())
        choices2 = filter2.get_choices(qs)
        self.assertEqual(len(choices2), 1)
        self.assertEqual(choices2[0].link_type, FILTER_DISPLAY)

    def test_manytomany_filter(self):
        """
        Tests for ManyToManyFilter
        """
        filter1 = ManyToManyFilter('authors', Book, MultiValueDict())
        qs = Book.objects.all()

        # ManyToMany can have 'drill down', i.e. multiple levels of filtering,
        # which can be removed individually.

        # First level:
        choices = filter1.get_choices(qs)

        # Check list is full, and in right order
        self.assertEqual([unicode(v) for v in Author.objects.all()],
                         [choice.label for choice in choices])

        for choice in choices:
            # For single choice, param will be single integer:
            param = int(choice.params[filter1.query_param])

            # Check the count
            count = Book.objects.filter(authors=int(param)).count()
            self.assertEqual(choice.count, count)

            author = Author.objects.get(id=param)

            # Check the label
            self.assertEqual(unicode(author),
                             choice.label)

            # Check the filtering
            filter2 = ManyToManyFilter('authors', Book, choice.params)
            qs_filtered = filter2.apply_filter(qs)
            self.assertEqual(len(qs_filtered), choice.count)

            for book in qs_filtered:
                self.assertTrue(author in book.authors.all())

            # Check we've got a 'remove link' on filtered.
            choices_filtered = filter2.get_choices(qs)
            self.assertEqual(choices_filtered[0].link_type, FILTER_REMOVE)

    def test_manytomany_filter_multiple(self):
        qs = Book.objects.all()

        # Specific example - multiple filtering
        emily = Author.objects.get(name='Emily Brontë')
        charlotte = Author.objects.get(name='Charlotte Brontë')
        anne = Author.objects.get(name='Anne Brontë')

        # If we select 'emily' as an author:

        data =  MultiValueDict({'authors':[str(emily.pk)]})
        filter1 = ManyToManyFilter('authors', Book, data)
        qs_emily = filter1.apply_filter(qs)

        # ...we should get a qs that includes Poems and Wuthering Heights.
        self.assertTrue(qs_emily.filter(name='Poems').exists())
        self.assertTrue(qs_emily.filter(name='Wuthering Heights').exists())
        # ...and excludes Jane Eyre
        self.assertFalse(qs_emily.filter(name='Jane Eyre').exists())

        # We should get a 'choices' that includes charlotte and anne
        choices = filter1.get_choices(qs_emily)
        self.assertTrue(unicode(anne) in [c.label for c in choices if c.link_type is FILTER_ADD])
        self.assertTrue(unicode(charlotte) in [c.label for c in choices if c.link_type is FILTER_ADD])

        # ... but not emily, because that is obvious and boring
        self.assertTrue(unicode(emily) not in [c.label for c in choices if c.link_type is FILTER_ADD])
        # emily should be in 'remove' links, however.
        self.assertTrue(unicode(emily) in [c.label for c in choices if c.link_type is FILTER_REMOVE])

        # If we select again:
        data2 =  MultiValueDict({'authors': [str(emily.pk), str(anne.pk)]})
        filter2 = ManyToManyFilter('authors', Book, data2)

        qs_emily_anne = filter2.apply_filter(qs)

        # ...we should get a qs that includes Poems
        self.assertTrue(qs_emily_anne.filter(name='Poems').exists())
        # ... but not Wuthering Heights
        self.assertFalse(qs_emily_anne.filter(name='Wuthering Heights').exists())

        # The choices should contain just emily and anne to remove, and
        # charlotte should have 'link_type' FILTER_ADD. Even though it
        # is the only choice, adding the choice is not necessarily the same as
        # not adding it (could have books by Rmily and Anne, but not charlotte)
        choices = filter2.get_choices(qs_emily_anne)
        self.assertEqual([(c.label, c.link_type) for c in choices],
                         [(unicode(emily), FILTER_REMOVE),
                          (unicode(anne), FILTER_REMOVE),
                          (unicode(charlotte), FILTER_ADD)])

    def test_manytomany_filter_invalid_query(self):
        self.do_invalid_query_param_test(lambda params:
                                             ManyToManyFilter('authors', Book, params),
                                         MultiValueDict({'authors':['xxx']}))
        self.do_missing_related_object_test(lambda params:
                                                ManyToManyFilter('authors', Book, params),
                                            MultiValueDict({'authors':['10000']}))

    def test_datetime_filter_multiple_year_choices(self):
        """
        Tests that DateTimeFilter can produce choices spanning a set of years
        (and limit to max_links)
        """
        # This does drill down, and has multiple values.
        f = DateTimeFilter('date_published', Book, MultiValueDict(), max_links=10)
        qs = Book.objects.all()

        # We have enough data that it will not show a simple list of years.
        choices = f.get_choices(qs)
        self.assertTrue(len(choices) <= 10)
        self.assertTrue('-' in choices[0].label)

    def test_datetime_filter_single_year_selected(self):
        params = MultiValueDict({'date_published':['1818']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__year=1818)))
        # We only need 1 query if we've already told it what year to look at.
        with self.assertNumQueries(1):
            choices = f.get_choices(qs_filtered)

        # There are at least 2 books in 1818, in different months.
        self.assertTrue(len([c for c in choices if c.link_type == FILTER_ADD]) >= 2)
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)

    def test_datetime_filter_year_range_selected(self):
        params = MultiValueDict({'date_published':['1813..1814']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        start = date(1813, 1, 1)
        end = date(1815, 1, 1)
        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__gte=start,
                                        date_published__lt=end)))

        # We only need 1 query if we've already told it what years to look at,
        # and there is data for both years.
        with self.assertNumQueries(1):
            choices = f.get_choices(qs_filtered)

        # There are at least 2 books in 1813..1814, on different years
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_ADD]), 2)
        self.assertEqual([c.label for c in choices if c.link_type == FILTER_ADD],
                         ['1813', '1814'])

    def test_datetime_filter_single_month_selected(self):
        params = MultiValueDict({'date_published':['1847-10']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__year=1847,
                                        date_published__month=10)))

        # We only need 1 query if we've already told it what month to look at.
        with self.assertNumQueries(1):
            choices = f.get_choices(qs_filtered)

        # There are at least 2 books for Oct 1847, on different days
        self.assertTrue(len([c for c in choices if c.link_type == FILTER_ADD]) >= 2)
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)

    def test_datetime_filter_month_range_selected(self):
        params = MultiValueDict({'date_published':['1818-08..1818-09']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        start = date(1818, 8, 1)
        end = date(1818, 10, 1)
        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__gte=start,
                                        date_published__lt=end)))

        # We only need 1 query if we've already told it what months to look at,
        # and there is data for both months.
        with self.assertNumQueries(1):
            choices = f.get_choices(qs_filtered)

        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)
        # There are at least 2 books in this range, in different months
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_ADD]), 2)
        self.assertEqual([c.label for c in choices if c.link_type == FILTER_ADD],
                         ['August', 'September'])

    def test_datetime_filter_single_day_selected(self):
        params = MultiValueDict({'date_published':['1847-10-16']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__year=1847,
                                        date_published__month=10,
                                        date_published__day=16)))

        # We need 0 queries if we've already told it what day to look at.
        with self.assertNumQueries(0):
            choices = f.get_choices(qs_filtered)

        # There can be no add links.
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_ADD]), 0)
        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)

    def test_datetime_filter_day_range_selected(self):
        params = MultiValueDict({'date_published':['1847-10-10..1847-10-23']})
        f = DateTimeFilter('date_published', Book, params, max_links=10)
        qs = Book.objects.all()

        # Should get a number of books in queryset.
        qs_filtered = f.apply_filter(qs)

        start = date(1847, 10, 10)
        end = date(1847, 10, 24)
        self.assertEqual(list(qs_filtered),
                         list(qs.filter(date_published__gte=start,
                                        date_published__lt=end)))

        # We only need 1 query if we've already told it what days to look at,
        # and there is data for more than one day.
        with self.assertNumQueries(1):
            choices = f.get_choices(qs_filtered)

        self.assertEqual(len([c for c in choices if c.link_type == FILTER_REMOVE]), 1)
        # There are at least 2 books in this range, on different days.
        add_choices = [c for c in choices if c.link_type == FILTER_ADD]
        self.assertTrue(len(add_choices) >= 2)

        self.assertTrue("16" in [c.label for c in add_choices])

    def test_datetime_filter_start_at_year(self):
        # Tests that the first filter shown is a year, not a day,
        # even if initial query gets you down to a day.
        params = MultiValueDict()
        qs = Book.objects.filter(id=1)
        f = DateTimeFilter('date_published', Book, params, max_links=10)

        choices = f.get_choices(qs)
        self.assertEqual(len(choices), 3)

        self.assertEqual(choices[0].link_type, FILTER_DISPLAY)
        self.assertEqual(choices[0].label, str(qs[0].date_published.year))

        self.assertEqual(choices[1].link_type, FILTER_DISPLAY)

        self.assertEqual(choices[2].link_type, FILTER_DISPLAY)
        self.assertEqual(choices[2].label, str(qs[0].date_published.day))

    def test_datetime_filter_select_year_display_month(self):
        # Tests that if a year is selected, and only one thing matches,
        # the month should be displayed in 'display' mode.
        qs = Book.objects.filter(id=1)
        params = MultiValueDict(dict(date_published=[str(qs[0].date_published.year)]))
        f = DateTimeFilter('date_published', Book, params, max_links=10)

        choices = f.get_choices(qs)
        self.assertEqual(len(choices), 2)

        self.assertEqual(choices[0].link_type, FILTER_REMOVE)
        self.assertEqual(choices[0].label, str(qs[0].date_published.year))

        self.assertEqual(choices[1].link_type, FILTER_DISPLAY)

    def test_datetime_filter_max_depth(self):
        qs = Book.objects.all()
        params = MultiValueDict({'date_published':['1813']})
        f = DateTimeFilter('date_published', Book, params, max_depth='year')
        choices = f.get_choices(f.apply_filter(qs))
        self.assertEqual(len(choices), 1)
        self.assertEqual(choices[0].link_type, FILTER_REMOVE)

    def test_datetime_filter_invalid_query(self):
        self.do_invalid_query_param_test(lambda params: DateTimeFilter('date_published', Book, params, max_links=10),
                                         MultiValueDict({'date_published':['1818xx']}))

    def test_datetime_filter_empty_qs(self):
        """
        Tests that DateTimeFilter works when it is passed in an empty QuerySet.
        """
        f = DateTimeFilter('date_published', Book, MultiValueDict(), max_links=10)
        qs = Book.objects.filter(id=1000)

        qs_filtered = f.apply_filter(qs)
        choices = f.get_choices(qs_filtered)
        self.assertEqual(len(choices), 0)
        self.assertEqual(len(qs_filtered), 0)

    def test_datetime_filter_remove_broad(self):
        """
        If we remove a broader choice (e.g. year), the more specific choices
        (e.g. day) should be removed too.
        """
        # This should hold whichever order the params are defined:
        params1 = MultiValueDict({'date_published': ['1818-08-24',
                                                     '1818-08-24..1818-08-30',
                                                     '1818-08',
                                                     '1818-08..1818-10',
                                                     '1818..1819',
                                                     '1818']})
        params2 = MultiValueDict({'date_published': ['1818..1819',
                                                     '1818',
                                                     '1818-08..1818-10',
                                                     '1818-08',
                                                     '1818-08-24..1818-08-30',
                                                     '1818-08-24',
                                                     ]})

        for p in [params1, params2]:
            f = DateTimeFilter('date_published', Book, p)
            qs = Book.objects.all()
            qs_filtered = f.apply_filter(qs)
            choices = f.get_choices(qs_filtered)

            # First choice should be for '1818-1819' and remove all 'date_published'
            self.assertEqual(choices[0].label, '1818-1819')
            self.assertEqual(choices[0].link_type, FILTER_REMOVE)
            self.assertEqual(choices[0].params.getlist('date_published'),
                             [])

            self.assertEqual(choices[1].link_type, FILTER_REMOVE)
            self.assertEqual(choices[1].params.getlist('date_published'),
                             ['1818..1819'])

            self.assertEqual(choices[2].link_type, FILTER_REMOVE)
            self.assertEqual(choices[2].params.getlist('date_published'),
                             ['1818..1819',
                              '1818',
                              ])

            self.assertEqual(choices[3].link_type, FILTER_REMOVE)
            self.assertEqual(choices[3].params.getlist('date_published'),
                             ['1818..1819',
                              '1818',
                              '1818-08..1818-10',
                              ])

            self.assertEqual(choices[4].link_type, FILTER_REMOVE)
            self.assertEqual(choices[4].params.getlist('date_published'),
                             ['1818..1819',
                              '1818',
                              '1818-08..1818-10',
                              '1818-08',
                              ])

            self.assertEqual(choices[5].link_type, FILTER_REMOVE)
            self.assertEqual(choices[5].params.getlist('date_published'),
                             ['1818..1819',
                              '1818',
                              '1818-08..1818-10',
                              '1818-08',
                              '1818-08-24..1818-08-30',
                              ])


    def test_numericrange_filter_simple_vals(self):
        # If data is less than max_links, we should get a simple list of values.
        filter1 = NumericRangeFilter('price', Book, MultiValueDict(), max_links=20)

        # Limit to single value to force the case
        qs = Book.objects.filter(price=Decimal('3.50'))

        # Should only take 2 queries - one to find out how many distinct values,
        # one to get the counts.
        with self.assertNumQueries(2):
            choices = filter1.get_choices(qs)

        self.assertEqual(len(choices), 1)
        self.assertTrue('3.5' in choices[0].label)

    def test_numericrange_filter_range_choices(self):
        # If data is more than max_links, we should get a range
        filter1 = NumericRangeFilter('price', Book, MultiValueDict(), max_links=8)

        qs = Book.objects.all()
        # Should take 3 queries - one to find out how many distinct values,
        # one to find a range, one to get the counts.
        with self.assertNumQueries(3):
            choices = filter1.get_choices(qs)

        self.assertTrue(len(choices) <= 8)
        total_count = sum(c.count for c in choices)
        self.assertEqual(total_count, qs.count())

        # First choice should be inclusive on first and last
        p0 = choices[0].params.getlist('price')[0]
        self.assertTrue('..' in p0)
        self.assertTrue('i' in p0.split('..')[0])
        self.assertTrue('i' in p0.split('..')[1])

        # Second choice should be exlusive on first,
        # inclusive on second.
        p1 = choices[1].params.getlist('price')[0]
        self.assertTrue('..' in p1)
        self.assertTrue('i' not in p1.split('..')[0])
        self.assertTrue('i' in p1.split('..')[1])

    def test_numericrange_filter_apply_filter(self):
        qs = Book.objects.all()

        # exclusive
        params1 = MultiValueDict({'price': ['3.50..4.00']})
        filter1 = NumericRangeFilter('price', Book, params1)
        qs_filtered1 = filter1.apply_filter(qs)
        self.assertEqual(list(qs_filtered1),
                         list(qs.filter(price__gt=Decimal('3.50'),
                                        price__lt=Decimal('4.00'))))

        # inclusive
        params2 = MultiValueDict({'price': ['3.50i..4.00i']})
        filter2 = NumericRangeFilter('price', Book, params2)
        qs_filtered2 = filter2.apply_filter(qs)
        self.assertEqual(list(qs_filtered2),
                         list(qs.filter(price__gte=Decimal('3.50'),
                                        price__lte=Decimal('4.00'))))

    def test_numericrange_filter_manual_ranges(self):
        """
        Test we can specify 'ranges' and it works as expected.
        """
        # Also tests that aggregation works as expected with regards to
        # lower/upper limits.
        qs = Book.objects.all()

        ranges = [(Decimal('3.50'), Decimal('5.00')),
                  (Decimal('5.00'), Decimal('6.00'))]
        # There are books with prices exactly equal to 3.50/5.00/6.00 which
        # makes this test real.

        self.assertTrue(qs.filter(price=Decimal('3.50')).exists())
        self.assertTrue(qs.filter(price=Decimal('5.00')).exists())
        self.assertTrue(qs.filter(price=Decimal('6.00')).exists())

        filter1 = NumericRangeFilter('price', Book, MultiValueDict(), ranges=ranges)
        choices = filter1.get_choices(qs)
        self.assertEqual(choices[0].count, qs.filter(price__gte=Decimal('3.50'), price__lte=Decimal('5.00')).count())
        self.assertEqual(choices[1].count, qs.filter(price__gt=Decimal('5.00'), price__lte=Decimal('6.00')).count())

    def test_numericrange_filter_manual_ranges_labels(self):
        """
        Test we can specify 'ranges' with manual labels
        """
        qs = Book.objects.all()

        ranges = [(Decimal('1.00'), Decimal('4.00'), "$4 or less"),
                  (Decimal('4.00'), Decimal('6.00'), "$4.00 - $6.00"),
                  (Decimal('6.00'), Decimal('100.00'), "More than $6")]

        filter1 = NumericRangeFilter('price', Book, MultiValueDict(), ranges=ranges)
        choices = filter1.get_choices(qs)
        self.assertEqual(choices[0].label, "$4 or less")
        self.assertEqual(choices[1].label, "$4.00 - $6.00")

    def test_numericrange_filter_drilldown(self):
        # Can specify to turn off drilldown
        # We shouldn't get drilldown if ranges is specified manually.

        params1 = MultiValueDict({'price': ['3.50i..5.00i']})
        filter1 = NumericRangeFilter('price', Book, params1, drilldown=False)

        qs = Book.objects.all()
        qs_filtered1 = filter1.apply_filter(qs)
        choices = filter1.get_choices(qs_filtered1)
        self.assertEqual(len(choices), 1)
        self.assertEqual(choices[0].link_type, FILTER_REMOVE)

    def test_order_by_count(self):
        """
        Tests the 'order_by_count' option.
        """
        filter1 = ForeignKeyFilter('genre', Book, MultiValueDict(), order_by_count=True)
        qs = Book.objects.all()
        choices1 = filter1.get_choices(qs)

        # Should be same after sorting by 'count'
        self.assertEqual(choices1, sorted(choices1, key=operator.attrgetter('count'), reverse=True))

        filter2 = ForeignKeyFilter('genre', Book, MultiValueDict(), order_by_count=False)
        choices2 = filter2.get_choices(qs)

        # Should be same after sorting by 'label' (that is equal to Genre.name,
        # and Genre ordering is by that field)
        self.assertEqual(choices2, sorted(choices2, key=operator.attrgetter('label')))



#!/usr/bin/env python
from setuptools import setup, find_packages
import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

def find_package_data(pkg, filetypes):
    import glob
    import itertools

    out = []
    for f in filetypes:
        for x in range(0, 20):
            pattern = pkg + '/' + ('*/' * x) + f
            out.extend([p[len(pkg)+1:] for p in glob.glob(pattern)])
    return out


setup(
    name = "django-easyfilters",
    version = '0.2',
    packages = find_packages(),
    author = "Luke Plant",
    author_email = "L.Plant.98@cantab.net",
    url = "https://bitbucket.org/spookylukey/django-easyfilters/",
    description = "Easy creation of link-based filtering for a list of Django model objects.",
    long_description = (
                        read('README.rst')
                        + "\n\n" +
                        read('CHANGES.rst')
    ),
    package_data = {
        'django_easyfilters': find_package_data('django_easyfilters', ['*.json', '*.html', '*.css', '*.js'])
        },
    license = "MIT",
    keywords = "django filter autofilter drilldown easy simple",
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Framework :: Django",
        "Topic :: Software Development :: User Interfaces",
        ],
    install_requires = ['django >= 1.3', 'python-dateutil'],
)




DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'tests.db',
    },
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.contenttypes',
    'django_easyfilters',
    'django_easyfilters.tests',
]

ROOT_URLCONF = 'django_easyfilters.tests.urls'

DEBUG = True

SITE_ID = 1

STATIC_URL = '/static/'



from django.shortcuts import render

from django_easyfilters.tests.models import Book, Author
from django_easyfilters import FilterSet
from django_easyfilters.filters import NumericRangeFilter

class BookFilterSet(FilterSet):
    fields = [
        'binding',
        'authors',
        'genre',
        'price',
        'date_published',
        ]

def books(request):
    books = Book.objects.all()
    booksfilter = BookFilterSet(books, request.GET)
    return render(request, "books.html", {'books': booksfilter.qs,
                                          'booksfilter': booksfilter})

class AuthorFilterSet(FilterSet):
    fields = [
        ('likes', {}, NumericRangeFilter)
        ]

def authors(request):
    authors = Author.objects.all()
    authorsfilter = AuthorFilterSet(authors, request.GET)
    return render(request, "authors.html", {'authors': authorsfilter.qs,
                                            'authorsfilter': authorsfilter})
